@@ -50,6 +50,8 @@
 package org.glassfish.grizzly.http.server;
 
 import java.io.IOException;
+import java.util.concurrent.Callable;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.logging.Level;
@@ -60,9 +62,11 @@ import org.glassfish.grizzly.EmptyCompletionHandler;
 import org.glassfish.grizzly.Grizzly;
 import org.glassfish.grizzly.ReadHandler;
 import org.glassfish.grizzly.attributes.Attribute;
+import org.glassfish.grizzly.filterchain.Filter;
 import org.glassfish.grizzly.filterchain.FilterChainContext;
 import org.glassfish.grizzly.filterchain.FilterChainEvent;
 import org.glassfish.grizzly.filterchain.NextAction;
+import org.glassfish.grizzly.filterchain.ShutdownEvent;
 import org.glassfish.grizzly.filterchain.TransportFilter;
 import org.glassfish.grizzly.http.HttpContent;
 import org.glassfish.grizzly.http.HttpContext;
@@ -72,8 +76,10 @@ import org.glassfish.grizzly.http.HttpResponsePacket;
 import org.glassfish.grizzly.http.Method;
 import org.glassfish.grizzly.http.server.util.HtmlHelper;
 import org.glassfish.grizzly.http.util.HttpStatus;
+import org.glassfish.grizzly.impl.FutureImpl;
 import org.glassfish.grizzly.localization.LogMessages;
 import org.glassfish.grizzly.utils.DelayedExecutor;
+import org.glassfish.grizzly.utils.Futures;
 import com.openexchange.exception.ExceptionUtils;
 import com.openexchange.http.grizzly.GrizzlyConfig;
 
@@ -113,11 +119,12 @@ public class OXHttpServerFilter extends HttpServerFilter {
     /**
      * The flag, which indicates if the server is currently in the shutdown phase
      */
-    private volatile boolean isShuttingDown;
+    private final AtomicBoolean shuttingDown = new AtomicBoolean();
+
     /**
      * CompletionHandler to be notified, when shutdown could be gracefully completed
      */
-    private AtomicReference<CompletionHandler<HttpServerFilter>> shutdownCompletionHandlerRef;
+    private AtomicReference<FutureImpl<HttpServerFilter>> shutdownCompletionFuture;
 
     /**
      * The number of requests, which are currently in process.
@@ -205,7 +212,7 @@ public class OXHttpServerFilter extends HttpServerFilter {
                 try {
                     ctx.setMessage(handlerResponse);
 
-                    if (isShuttingDown) { // if we're in the shutting down phase - serve shutdown page and exit
+                    if (shuttingDown.get()) { // if we're in the shutting down phase - serve shutdown page and exit
                         handlerResponse.getResponse().getProcessingState().setError(true);
                         HtmlHelper.setErrorAndSendErrorPage(
                                 handlerRequest, handlerResponse,
@@ -317,26 +324,28 @@ public class OXHttpServerFilter extends HttpServerFilter {
         }
     }
 
-
-    // ---------------------------------------------------------- Public Methods
-
-    /**
-     * Method, which might be optionally called to prepare the filter for
-     * shutdown.
-     * @param shutdownCompletionHandler {@link CompletionHandler} to be notified,
-     *        when shutdown could be gracefully completed
-     */
     @Override
-    public void prepareForShutdown(
-            final CompletionHandler<HttpServerFilter> shutdownCompletionHandler) {
-        this.shutdownCompletionHandlerRef =
-                new AtomicReference<CompletionHandler<HttpServerFilter>>(shutdownCompletionHandler);
-        isShuttingDown = true;
+    public NextAction handleEvent(final FilterChainContext ctx, final FilterChainEvent event) throws IOException {
+        if (event.type() == ShutdownEvent.TYPE) {
+            if (shuttingDown.compareAndSet(false, true)) {
+                final ShutdownEvent shutDownEvent = (ShutdownEvent) event;
+                final FutureImpl<HttpServerFilter> future = Futures.createSafeFuture();
 
-        if (activeRequestsCounter.get() == 0 &&
-                shutdownCompletionHandlerRef.getAndSet(null) != null) {
-            shutdownCompletionHandler.completed(this);
+
+                shutDownEvent.addShutdownTask(new Callable<Filter>() {
+                    @Override
+                    public Filter call() throws Exception {
+                        return future.get();
+                    }
+                });
+
+                shutdownCompletionFuture = new AtomicReference<>(future);
+                if (activeRequestsCounter.get() == 0) {
+                    future.result(this);
+                }
+            }
         }
+        return ctx.getInvokeAction();
     }
 
     // --------------------------------------------------------- Private Methods
@@ -387,14 +396,14 @@ public class OXHttpServerFilter extends HttpServerFilter {
      */
     private void onRequestCompleteAndResponseFlushed() {
         final int count = activeRequestsCounter.decrementAndGet();
-        if (count == 0 && isShuttingDown) {
-            final CompletionHandler<HttpServerFilter> shutdownHandler =
-                    shutdownCompletionHandlerRef != null
-                    ? shutdownCompletionHandlerRef.getAndSet(null)
-                    : null;
+        if (count == 0 && shuttingDown.get()) {
+            final FutureImpl<HttpServerFilter> shutdownFuture =
+                shutdownCompletionFuture != null
+                ? shutdownCompletionFuture.getAndSet(null)
+                : null;
 
-            if (shutdownHandler != null) {
-                shutdownHandler.completed(this);
+            if (shutdownFuture != null) {
+                shutdownFuture.result(this);
             }
         }
     }
